@@ -1,5 +1,5 @@
 /*******************************************************************************
-    Copyright (c) 2014-2022 NVidia Corporation
+    Copyright (c) 2014-2023 NVidia Corporation
 
     Permission is hereby granted, free of charge, to any person obtaining a copy
     of this software and associated documentation files (the "Software"), to
@@ -108,7 +108,7 @@ struct nvlink_device
     // Device type and status
     NvU64  type;
     NvBool initialized;
-
+    
     // Training type: ALI or Non-ALI
     NvBool enableALI;
 
@@ -120,6 +120,12 @@ struct nvlink_device
     NvU32 numLinksPerIoctrl;
     NvU32 numActiveLinksPerIoctrl;
 
+    //
+    // boolean indicating if a given device
+    // is a reduced nvlink config
+    //
+    NvBool bReducedNvlinkConfig;
+
     // Client private information
     void *pDevInfo;
 };
@@ -141,6 +147,20 @@ struct nvlink_link_change
     enum nvlink_link_change_type change_type;
 };
 
+//
+// Structure representing Nvlink Error Threshold
+//
+struct nvlink_link_error_threshold
+{
+    NvU8 thresholdMan;
+    NvU8 thresholdExp;
+    NvU8 timescaleMan;
+    NvU8 timescaleExp;
+    NvBool bInterruptEn;
+    NvBool bUserConfig;
+    NvBool bInterruptTrigerred; // Error threshold interrupt generated
+};
+
 // nvlink link state
 struct nvlink_link
 {
@@ -233,6 +253,8 @@ struct nvlink_link
 
     //seed data for given nvlink
     NvU32 seedData[NVLINK_MAX_SEED_BUFFER_SIZE];
+
+    struct nvlink_link_error_threshold errorThreshold;
 };
 
 // nvlink link handler ops
@@ -328,7 +350,7 @@ typedef struct nvlink_inband_data      nvlink_inband_data;
 #define NVLINK_LINKSTATE_CONTAIN                        0x19   // TL is in contain mode
 #define NVLINK_LINKSTATE_INITTL                         0x1A   // INITTL
 #define NVLINK_LINKSTATE_INITPHASE5                     0x1B   // INITPHASE5
-#define NVLINK_LINKSTATE_ALI                            0x1C   // ALI
+#define NVLINK_LINKSTATE_ALI                            0x1C   // ALI 
 #define NVLINK_LINKSTATE_ACTIVE_PENDING                 0x1D   // Intermediate state for a link going to active
 #define NVLINK_LINKSTATE_INVALID                        0xFF   // Invalid state
 
@@ -381,6 +403,10 @@ NvBool nvlink_lib_is_initialized(void);
  */
 NvBool nvlink_lib_is_device_list_empty(void);
 
+/*
+ * Get if a device registerd to the nvlink corelib has a reduced nvlink config
+ */
+NvBool nvlink_lib_is_registerd_device_with_reduced_config(void);
 
 /************************************************************************************************/
 /************************** NVLink library driver-side interface ********************************/
@@ -449,6 +475,7 @@ NvlStatus nvlink_lib_is_link_using_ALI(nvlink_link *link, NvBool *usingALI);
  * Set the training state for the given link as non-ALI or ALI
  */
 NvlStatus nvlink_lib_link_set_training_mode(nvlink_link *link, NvBool enableALI);
+
 /************************************************************************************************/
 /*************************** NVLink topology discovery functions ********************************/
 /************************************************************************************************/
