@@ -71,6 +71,7 @@ static void CalculateVideoInfoFrameColorFormat(
     // sets video infoframe colorspace (RGB/YUV).
     switch (pAttributesSet->colorSpace) {
     case NV_KMS_DPY_ATTRIBUTE_CURRENT_COLOR_SPACE_RGB:
+    case NV_KMS_DPY_ATTRIBUTE_CURRENT_COLOR_SPACE_BT2020RGB:
         pCtrl->color_space = NVT_VIDEO_INFOFRAME_BYTE1_Y1Y0_RGB;
         break;
     case NV_KMS_DPY_ATTRIBUTE_CURRENT_COLOR_SPACE_YCbCr422:
@@ -92,6 +93,9 @@ static void CalculateVideoInfoFrameColorFormat(
     case NV_KMS_DPY_ATTRIBUTE_CURRENT_COLOR_SPACE_RGB:
         pCtrl->colorimetry = NVT_COLORIMETRY_RGB;
         break;
+    case NV_KMS_DPY_ATTRIBUTE_CURRENT_COLOR_SPACE_BT2020RGB:
+        pCtrl->colorimetry = NVT_COLORIMETRY_BT2020RGB;
+        break;
     case NV_KMS_DPY_ATTRIBUTE_CURRENT_COLOR_SPACE_YCbCr422:
     case NV_KMS_DPY_ATTRIBUTE_CURRENT_COLOR_SPACE_YCbCr444:
         if (hdTimings) {
@@ -123,9 +127,13 @@ static void CalculateVideoInfoFrameColorFormat(
         break;
     }
 
-    // Only limited color range is allowed with YUV444 or YUV422 color spaces
+    /*
+     * Only limited color range is allowed with YUV444, YUV422 color spaces, or
+     * BT2020 colorimetry.
+     */
     nvAssert(!(((pCtrl->color_space == NVT_VIDEO_INFOFRAME_BYTE1_Y1Y0_YCbCr422) ||
-                (pCtrl->color_space == NVT_VIDEO_INFOFRAME_BYTE1_Y1Y0_YCbCr444)) &&
+                (pCtrl->color_space == NVT_VIDEO_INFOFRAME_BYTE1_Y1Y0_YCbCr444) ||
+                (pCtrl->colorimetry == NVT_COLORIMETRY_BT2020RGB)) &&
                (pCtrl->rgb_quantization_range !=
                     NVT_VIDEO_INFOFRAME_BYTE3_Q1Q0_LIMITED_RANGE)));
 }
@@ -434,11 +442,11 @@ static void SendInfoFrame(const NVDispEvoRec *pDispEvo,
 static void SendVideoInfoFrame(const NVDispEvoRec *pDispEvo,
                                const NvU32 head,
                                const NVAttributesSetEvoRec *pAttributesSet,
-                               const NvBool hdTimings,
-                               const NVT_VIDEO_INFOFRAME_CTRL *pCtrl,
+                               const NVDispHeadInfoFrameStateEvoRec *pInfoFrameState,
                                NVT_EDID_INFO *pEdidInfo)
 {
-    NVT_VIDEO_INFOFRAME_CTRL videoCtrl = *pCtrl;
+    NvBool hdTimings = pInfoFrameState->hdTimings;
+    NVT_VIDEO_INFOFRAME_CTRL videoCtrl = pInfoFrameState->ctrl;
     NVT_VIDEO_INFOFRAME VideoInfoFrame;
     NVT_STATUS status;
 
@@ -519,8 +527,7 @@ SendHDMI3DVendorSpecificInfoFrame(const NVDispEvoRec *pDispEvo,
 void nvUpdateHdmiInfoFrames(const NVDispEvoRec *pDispEvo,
                             const NvU32 head,
                             const NVAttributesSetEvoRec *pAttributesSet,
-                            const NvBool hdTimings,
-                            const NVT_VIDEO_INFOFRAME_CTRL *pCtrl,
+                            const NVDispHeadInfoFrameStateEvoRec *pInfoFrameState,
                             NVDpyEvoRec *pDpyEvo)
 {
     if (!nvDpyIsHdmiEvo(pDpyEvo)) {
@@ -537,8 +544,7 @@ void nvUpdateHdmiInfoFrames(const NVDispEvoRec *pDispEvo,
     SendVideoInfoFrame(pDispEvo,
                        head,
                        pAttributesSet,
-                       hdTimings,
-                       pCtrl,
+                       pInfoFrameState,
                        &pDpyEvo->parsedEdid.info);
 
     SendHDMI3DVendorSpecificInfoFrame(pDispEvo,
@@ -1860,6 +1866,8 @@ NvBool nvHdmiFrlAssessLink(NVDpyEvoPtr pDpyEvo)
     NVHDMIPKT_RESULT ret;
     const NvU32 displayId = nvDpyIdToNvU32(pDpyEvo->pConnectorEvo->displayId);
 
+    nvAssert(nvDpyIsHdmiEvo(pDpyEvo));
+
     /* HDMI dpys not dynamic dpy so its connector should have a dpyId. */
     nvAssert(displayId != 0);
     nvAssert(pDpyEvo->parsedEdid.valid);
@@ -1878,33 +1886,54 @@ NvBool nvHdmiFrlAssessLink(NVDpyEvoPtr pDpyEvo)
     return pDpyEvo->hdmi.sinkCaps.linkMaxFRLRate != HDMI_FRL_DATA_RATE_NONE;
 }
 
-/* Determine if HDMI FRL is needed to drive the given timings on the given dpy. */
-static NvBool TimingsNeedFRL(const NVDpyEvoRec *pDpyEvo,
-                             const NVHwModeTimingsEvo *pTimings)
+/*
+ * Determine if the given HDMI dpy supports FRL.
+ *
+ * Returns TRUE if the dpy supports FRL, or FALSE otherwise.
+ */
+NvBool nvHdmiDpySupportsFrl(const NVDpyEvoRec *pDpyEvo)
 {
+    NvU32 passiveDpDongleMaxPclkKHz;
     const NVDevEvoRec *pDevEvo = pDpyEvo->pDispEvo->pDevEvo;
 
-    /* Can't use FRL if the display hardware doesn't support it */
+    nvAssert(nvDpyIsHdmiEvo(pDpyEvo));
+
+    /* Can't use FRL if the display hardware doesn't support it. */
     if (!pDevEvo->hal->caps.supportsHDMIFRL) {
         return FALSE;
     }
 
-    /* Can only use FRL for HDMI devices. */
-    if (!nvDpyIsHdmiEvo(pDpyEvo)) {
-        return FALSE;
-    }
-
-    /* Can only use FRL if the HDMI sink supports it. */
+    /* Can't use FRL if the HDMI sink doesn't support it. */
     if (!pDpyEvo->parsedEdid.valid ||
         !pDpyEvo->parsedEdid.info.hdmiForumInfo.max_FRL_Rate) {
         return FALSE;
     }
 
+    /* Can't use FRL if we are using a passive DP to HDMI dongle. */
+    if (nvDpyGetPassiveDpDongleType(pDpyEvo, &passiveDpDongleMaxPclkKHz) !=
+        NV_EVO_PASSIVE_DP_DONGLE_UNUSED) {
+        return FALSE;
+    }
+
+    return TRUE;
+}
+
+/*
+ * Determine if HDMI FRL is needed to drive timings with the given pixel clock
+ * on the given dpy.
+ *
+ * Returns TRUE if FRL is needed, or FALSE otherwise.
+ * */
+static NvBool HdmiTimingsNeedFrl(const NVDpyEvoRec *pDpyEvo,
+                                 const NvU32 pixelClock)
+{
+    nvAssert(nvDpyIsHdmiEvo(pDpyEvo));
+
     /*
      * For HDMI, maxSingleLinkPixelClockKHz is the maximum non-FRL rate.
      * If the rate is higher than that, try to use FRL for the mode.
      */
-    return pTimings->pixelClock > pDpyEvo->maxSingleLinkPixelClockKHz;
+    return pixelClock > pDpyEvo->maxSingleLinkPixelClockKHz;
 }
 
 NvBool nvHdmiFrlQueryConfig(
@@ -1921,10 +1950,15 @@ NvBool nvHdmiFrlQueryConfig(
     NVT_TIMING nvtTiming = { };
     NVHDMIPKT_RESULT ret;
 
-    if (!TimingsNeedFRL(pDpyEvo, pHwTimings)) {
+    if (!nvDpyIsHdmiEvo(pDpyEvo) ||
+        !HdmiTimingsNeedFrl(pDpyEvo, pHwTimings->pixelClock)) {
         return TRUE;
     }
 
+    if (!nvHdmiDpySupportsFrl(pDpyEvo)) {
+        return FALSE;
+    }
+
     /* See if we can find an NVT_TIMING for this mode from the EDID. */
     pNvtTiming = nvFindEdidNVT_TIMING(pDpyEvo, pModeTimings, pValidationParams);
 
