@@ -126,8 +126,8 @@ gisubscriptionConstruct_IMPL
     }
 
     //
-    // Root-SwizzID is a special swizzID which doesn't have any GPU instance 
-    // associated with it. It can be subscribed to even without GPU instances 
+    // Root-SwizzID is a special swizzID which doesn't have any GPU instance
+    // associated with it. It can be subscribed to even without GPU instances
     //
     if (swizzId == NVC637_DEVICE_PROFILING_SWIZZID)
     {
@@ -231,7 +231,7 @@ gisubscriptionCopyConstruct_IMPL
     OBJGPU *pGpu = GPU_RES_GET_GPU(pGPUInstanceSubscription);
 
     {
-        // non kernel clients are not allowed to dup GPU instances 
+        // non kernel clients are not allowed to dup GPU instances
         NV_CHECK_OR_RETURN(LEVEL_SILENT, pCallContext->secInfo.privLevel >= RS_PRIV_LEVEL_KERNEL,
                            NV_ERR_NOT_SUPPORTED);
     }
@@ -360,7 +360,7 @@ gisubscriptionCtrlCmdExecPartitionsCreate_IMPL
     KernelMIGManager *pKernelMIGManager = GPU_GET_KERNEL_MIG_MANAGER(pGpu);
     KERNEL_MIG_GPU_INSTANCE *pKernelMIGGpuInstance = pGPUInstanceSubscription->pKernelMIGGpuInstance;
 
-    LOCK_ASSERT_AND_RETURN(rmApiLockIsOwner() && rmGpuLockIsOwner());
+    LOCK_ASSERT_AND_RETURN(rmapiLockIsOwner() && rmGpuLockIsOwner());
 
 {
     CALL_CONTEXT *pCallContext = resservGetTlsCallContext();
@@ -405,13 +405,13 @@ gisubscriptionCtrlCmdExecPartitionsCreate_IMPL
         {
             .type = KMIGMGR_CREATE_COMPUTE_INSTANCE_PARAMS_TYPE_REQUEST,
             .inst.request.count = pParams->execPartCount,
-            .inst.request.pReqComputeInstanceInfo = pParams->execPartInfo
+            .inst.request.pReqComputeInstanceInfo = pParams->execPartInfo,
+            .inst.request.requestFlags = pParams->flags
         };
 
-        if (hypervisorIsVgxHyper() &&
-            FLD_TEST_REF(NVC637_CTRL_DMA_EXEC_PARTITIONS_CREATE_REQUEST_WITH_PART_ID, _TRUE, pParams->flags))
+        if (!hypervisorIsVgxHyper())
         {
-            request.type = KMIGMGR_CREATE_COMPUTE_INSTANCE_PARAMS_TYPE_REQUEST_WITH_IDS;
+            request.inst.request.requestFlags = FLD_SET_DRF(C637_CTRL, _DMA_EXEC_PARTITIONS_CREATE_REQUEST, _WITH_PART_ID, _FALSE, request.inst.request.requestFlags);
         }
 
         if (IS_VIRTUAL(pGpu))
@@ -439,7 +439,7 @@ gisubscriptionCtrlCmdExecPartitionsCreate_IMPL
             KMIGMGR_CREATE_COMPUTE_INSTANCE_PARAMS restore =
             {
                 .type = KMIGMGR_CREATE_COMPUTE_INSTANCE_PARAMS_TYPE_RESTORE,
-                .inst.restore.pComputeInstanceSave = &save, 
+                .inst.restore.pComputeInstanceSave = &save,
             };
             portMemSet(&export, 0, sizeof(export));
             export.id = pParams->execPartId[i];
@@ -465,7 +465,7 @@ gisubscriptionCtrlCmdExecPartitionsCreate_IMPL
     }
 
     //
-    // Generate a subdevice event stating something has changed in GPU instance 
+    // Generate a subdevice event stating something has changed in GPU instance
     // config. Clients currently do not care about changes and their scope
     //
     if (!pParams->bQuery)
@@ -495,7 +495,7 @@ gisubscriptionCtrlCmdExecPartitionsDelete_IMPL
     KERNEL_MIG_GPU_INSTANCE *pKernelMIGGpuInstance = pGPUInstanceSubscription->pKernelMIGGpuInstance;
     NvU32 execPartIdx;
 
-    LOCK_ASSERT_AND_RETURN(rmApiLockIsOwner() && rmGpuLockIsOwner());
+    LOCK_ASSERT_AND_RETURN(rmapiLockIsOwner() && rmGpuLockIsOwner());
 
 {
     CALL_CONTEXT *pCallContext = resservGetTlsCallContext();
@@ -522,7 +522,7 @@ gisubscriptionCtrlCmdExecPartitionsDelete_IMPL
     // Check for trivial arguments
     NV_CHECK_OR_RETURN(LEVEL_SILENT, pParams->execPartCount > 0, NV_WARN_NOTHING_TO_DO);
 
-    // Check that the passed indices are valid compute instances 
+    // Check that the passed indices are valid compute instances
     for (execPartIdx = 0; execPartIdx < pParams->execPartCount; ++execPartIdx)
     {
         NvU32 execPartId = pParams->execPartId[execPartIdx];
@@ -551,7 +551,7 @@ gisubscriptionCtrlCmdExecPartitionsDelete_IMPL
     }
 
     //
-    // Generate a subdevice event stating something has changed in GPU instance 
+    // Generate a subdevice event stating something has changed in GPU instance
     // config. Clients currently do not care about changes and their scope
     //
     gpuNotifySubDeviceEvent(pGpu, NV2080_NOTIFIERS_SMC_CONFIG_UPDATE, NULL, 0, 0, 0);
@@ -592,17 +592,21 @@ gisubscriptionCtrlCmdExecPartitionsGet_IMPL
     NvU32 ciIdx;
     NvHandle hClient = RES_GET_CLIENT_HANDLE(pGPUInstanceSubscription);
     CALL_CONTEXT *pCallContext = resservGetTlsCallContext();
+    NvBool bEnumerateAll = NV_FALSE;
 
     NV_ASSERT_OR_RETURN(pCallContext != NULL, NV_ERR_INVALID_STATE);
-    NV_ASSERT_OR_RETURN(RMCFG_FEATURE_KERNEL_RM, NV_ERR_NOT_SUPPORTED);
 
-    NvBool bEnumerateAll = rmclientIsCapableOrAdminByHandle(hClient, 
-                                                            NV_RM_CAP_SYS_SMC_CONFIG,
-                                                            pCallContext->secInfo.privLevel);
+    // Capability checks shouldn't be done on
+    if (!RMCFG_FEATURE_PLATFORM_GSP)
+    {
+        bEnumerateAll = rmclientIsCapableOrAdminByHandle(hClient,
+                                                         NV_RM_CAP_SYS_SMC_CONFIG,
+                                                         pCallContext->secInfo.privLevel);
+    }
 
     MIG_COMPUTE_INSTANCE *pTargetComputeInstanceInfo = NULL;
 
-    LOCK_ASSERT_AND_RETURN(rmApiLockIsOwner() && rmGpuLockIsOwner());
+    LOCK_ASSERT_AND_RETURN(rmapiLockIsOwner() && rmGpuLockIsOwner());
 
     NV_ASSERT_OR_RETURN(pGpu->getProperty(pGpu, PDB_PROP_GPU_MIG_SUPPORTED),
                         NV_ERR_NOT_SUPPORTED);
@@ -612,7 +616,6 @@ gisubscriptionCtrlCmdExecPartitionsGet_IMPL
     (void)cisubscriptionGetComputeInstanceSubscription(RES_GET_CLIENT(pGPUInstanceSubscription), RES_GET_HANDLE(pGPUInstanceSubscription), &pComputeInstanceSubscription);
     if (pComputeInstanceSubscription != NULL)
     {
-        bEnumerateAll = NV_FALSE;
         pTargetComputeInstanceInfo = pComputeInstanceSubscription->pMIGComputeInstance;
     }
     else if (!bEnumerateAll)
@@ -642,17 +645,20 @@ gisubscriptionCtrlCmdExecPartitionsGet_IMPL
         pOutInfo->gpcCount = pMIGComputeInstance->resourceAllocation.gpcCount;
         pOutInfo->veidCount = pMIGComputeInstance->resourceAllocation.veidCount;
         pOutInfo->ceCount = kmigmgrCountEnginesOfType(&pMIGComputeInstance->resourceAllocation.engines,
-                                                      NV2080_ENGINE_TYPE_COPY(0));
+                                                      RM_ENGINE_TYPE_COPY(0));
         pOutInfo->nvEncCount = kmigmgrCountEnginesOfType(&pMIGComputeInstance->resourceAllocation.engines,
-                                                      NV2080_ENGINE_TYPE_NVENC(0));
+                                                      RM_ENGINE_TYPE_NVENC(0));
         pOutInfo->nvDecCount = kmigmgrCountEnginesOfType(&pMIGComputeInstance->resourceAllocation.engines,
-                                                      NV2080_ENGINE_TYPE_NVDEC(0));
+                                                      RM_ENGINE_TYPE_NVDEC(0));
         pOutInfo->nvJpgCount = kmigmgrCountEnginesOfType(&pMIGComputeInstance->resourceAllocation.engines,
-                                                      NV2080_ENGINE_TYPE_NVJPG);
+                                                      RM_ENGINE_TYPE_NVJPG);
         pOutInfo->ofaCount = kmigmgrCountEnginesOfType(&pMIGComputeInstance->resourceAllocation.engines,
-                                                      NV2080_ENGINE_TYPE_OFA);
+                                                      RM_ENGINE_TYPE_OFA);
         pOutInfo->sharedEngFlag = pMIGComputeInstance->sharedEngFlag;
         pOutInfo->veidStartOffset = pMIGComputeInstance->resourceAllocation.veidOffset;
+        pOutInfo->smCount = pMIGComputeInstance->resourceAllocation.smCount;
+        pOutInfo->computeSize = pMIGComputeInstance->computeSize;
+        pOutInfo->spanStart = pMIGComputeInstance->spanStart;
     }
 
     return status;
@@ -676,7 +682,7 @@ gisubscriptionCtrlCmdExecPartitionsGetActiveIds_IMPL
     KERNEL_MIG_GPU_INSTANCE *pKernelMIGGpuInstance = pGPUInstanceSubscription->pKernelMIGGpuInstance;
     NvU32 ciIdx;
 
-    LOCK_ASSERT_AND_RETURN(rmApiLockIsOwner() && rmGpuLockIsOwner());
+    LOCK_ASSERT_AND_RETURN(rmapiLockIsOwner() && rmGpuLockIsOwner());
 
     NV_ASSERT_OR_RETURN(pGpu->getProperty(pGpu, PDB_PROP_GPU_MIG_SUPPORTED),
                         NV_ERR_NOT_SUPPORTED);
@@ -774,6 +780,10 @@ gisubscriptionCtrlCmdExecPartitionsExport_IMPL
     pParams->info.sharedEngFlags = pMIGComputeInstance->sharedEngFlag;
     pParams->info.veidOffset     = pMIGComputeInstance->resourceAllocation.veidOffset;
     pParams->info.veidCount      = pMIGComputeInstance->resourceAllocation.veidCount;
+    pParams->info.smCount        = pMIGComputeInstance->resourceAllocation.smCount;
+    pParams->info.spanStart      = pMIGComputeInstance->spanStart;
+    pParams->info.computeSize    = pMIGComputeInstance->computeSize;
+
     for (gpcIdx = 0; gpcIdx < pMIGComputeInstance->resourceAllocation.gpcCount; ++gpcIdx)
     {
          pParams->info.gpcMask |= NVBIT32(pMIGComputeInstance->resourceAllocation.gpcIds[gpcIdx]);
@@ -840,7 +850,7 @@ gisubscriptionCtrlCmdExecPartitionsImport_IMPL
         KMIGMGR_CREATE_COMPUTE_INSTANCE_PARAMS restore =
         {
             .type = KMIGMGR_CREATE_COMPUTE_INSTANCE_PARAMS_TYPE_RESTORE,
-            .inst.restore.pComputeInstanceSave = &save, 
+            .inst.restore.pComputeInstanceSave = &save,
         };
 
         portMemSet(&save, 0, sizeof(save));
@@ -879,7 +889,7 @@ cleanup_rpc:
                             RES_GET_HANDLE(pGPUInstanceSubscription),
                             NVC637_CTRL_CMD_EXEC_PARTITIONS_DELETE,
                             &params,
-                            sizeof(params))); 
+                            sizeof(params)));
     }
 
     return status;
@@ -1006,3 +1016,152 @@ done:
     return;
 }
 
+NV_STATUS
+gisubscriptionCtrlCmdExecPartitionsGetProfileCapacity_IMPL
+(
+    GPUInstanceSubscription *pGPUInstanceSubscription,
+    NVC637_CTRL_EXEC_PARTITIONS_GET_PROFILE_CAPACITY_PARAMS *pParams
+)
+{
+    OBJGPU *pGpu = GPU_RES_GET_GPU(pGPUInstanceSubscription);
+    KERNEL_MIG_GPU_INSTANCE *pKernelMIGGpuInstance = pGPUInstanceSubscription->pKernelMIGGpuInstance;
+    KernelMIGManager *pKernelMIGManager = GPU_GET_KERNEL_MIG_MANAGER(pGpu);
+    NvBool bCtsRequired = kmigmgrIsCTSAlignmentRequired(pGpu, pKernelMIGManager);
+
+    NV_CHECK_OR_RETURN(LEVEL_ERROR,
+        pParams->computeSize < NV2080_CTRL_GPU_PARTITION_FLAG_COMPUTE_SIZE__SIZE,
+        NV_ERR_INVALID_ARGUMENT);
+
+    if (bCtsRequired)
+    {
+        NV_RANGE totalRange = kmigmgrComputeProfileSizeToCTSIdRange(pParams->computeSize);
+        NvU32 slotBasisComputeSize = kmigmgrSmallestComputeProfileSize(pGpu, pKernelMIGManager);
+        NvU64 slotBasisMask;
+        NvU64 validQueryMask;
+        NvU64 inUseIdMask;
+        NvU64 ctsId;
+        NvU32 totalSpanCount;
+        NvU32 availableSpanCount;
+        NV_RANGE slotBasisIdRange;
+
+        NV_CHECK_OR_RETURN(LEVEL_ERROR, slotBasisComputeSize != KMIGMGR_COMPUTE_SIZE_INVALID, NV_ERR_INVALID_STATE);
+
+        slotBasisIdRange = kmigmgrComputeProfileSizeToCTSIdRange(slotBasisComputeSize);
+
+        NV_CHECK_OR_RETURN(LEVEL_ERROR, !rangeIsEmpty(totalRange), NV_ERR_INVALID_ARGUMENT);
+        NV_CHECK_OR_RETURN(LEVEL_ERROR, !rangeIsEmpty(slotBasisIdRange), NV_ERR_INVALID_ARGUMENT);
+
+        slotBasisMask = DRF_SHIFTMASK64(slotBasisIdRange.hi:slotBasisIdRange.lo);
+        validQueryMask = DRF_SHIFTMASK64(totalRange.hi:totalRange.lo) & pKernelMIGGpuInstance->pProfile->validCTSIdMask;
+
+        // Find mask of un-usable IDs due to current in-use CTS Ids
+        inUseIdMask = 0x0;
+        FOR_EACH_INDEX_IN_MASK(64, ctsId, pKernelMIGGpuInstance->ctsIdsInUseMask)
+        {
+            NvU64 invalidMask;
+
+            NV_ASSERT_OK(kmigmgrGetInvalidCTSIdMask(pGpu, pKernelMIGManager, ctsId, &invalidMask));
+
+            inUseIdMask |= invalidMask;
+        }
+        FOR_EACH_INDEX_IN_MASK_END;
+
+        //
+        // The slot basis defines the smallest divison of the GPU instance.
+        // CTS IDs from this range are used as a means to specify span placements
+        // for compute profiles.
+        //
+        totalSpanCount = 0;
+        availableSpanCount = 0;
+
+        FOR_EACH_INDEX_IN_MASK(64, ctsId, validQueryMask)
+        {
+            NvU64 invalidMask;
+
+            NV_ASSERT_OK(kmigmgrGetInvalidCTSIdMask(pGpu, pKernelMIGManager, ctsId, &invalidMask));
+
+            invalidMask &= slotBasisMask;
+            pParams->totalSpans[totalSpanCount].lo = portUtilCountTrailingZeros64(invalidMask) - slotBasisIdRange.lo;
+            pParams->totalSpans[totalSpanCount].hi = nvPopCount64(invalidMask) + pParams->totalSpans[totalSpanCount].lo - 1;
+
+            if (!(NVBIT64(ctsId) & inUseIdMask))
+            {
+                pParams->availableSpans[availableSpanCount].lo = pParams->totalSpans[totalSpanCount].lo;
+                pParams->availableSpans[availableSpanCount].hi = pParams->totalSpans[totalSpanCount].hi;
+                availableSpanCount++;
+            }
+            totalSpanCount++;
+        }
+        FOR_EACH_INDEX_IN_MASK_END;
+
+        pParams->totalSpansCount     = totalSpanCount;
+        pParams->totalProfileCount   = totalSpanCount;
+        pParams->availableSpansCount = availableSpanCount;
+        pParams->profileCount        = availableSpanCount;
+    }
+    else
+    {
+        KernelGraphicsManager *pKernelGraphicsManager = GPU_GET_KERNEL_GRAPHICS_MANAGER(pGpu);
+        NV2080_CTRL_INTERNAL_MIGMGR_COMPUTE_PROFILE profile;
+        NvU64 veidMask;
+        NvU32 GPUInstanceVeidEnd;
+        NvU64 GPUInstanceVeidMask;
+        NvU64 GPUInstanceFreeVeidMask;
+        NvU64 GPUInstancePseudoMask;
+        NvU32 availableSpanCount;
+        NvU32 totalSpanCount;
+        NvU32 veidStepSize;
+        NvU32 veidSlotCount;
+        NvU32 count;
+        NvU32 i;
+
+        NV_CHECK_OK_OR_RETURN(LEVEL_ERROR,
+            kmigmgrGetComputeProfileFromSize(pGpu, pKernelMIGManager, pParams->computeSize, &profile));
+        NV_ASSERT_OK_OR_RETURN(
+            kgrmgrGetMaxVeidsPerGpc(pGpu, pKernelGraphicsManager, &veidStepSize));
+
+        // Create a mask for VEIDs associated with this GPU instance
+        veidMask = DRF_SHIFTMASK64(profile.veidCount - 1:0);
+        GPUInstanceVeidEnd = pKernelMIGGpuInstance->resourceAllocation.veidOffset + pKernelMIGGpuInstance->resourceAllocation.veidCount - 1;
+        GPUInstanceVeidMask = DRF_SHIFTMASK64(GPUInstanceVeidEnd:pKernelMIGGpuInstance->resourceAllocation.veidOffset);
+        GPUInstanceFreeVeidMask = GPUInstanceVeidMask & ~pKernelGraphicsManager->veidInUseMask;
+        GPUInstancePseudoMask = GPUInstanceFreeVeidMask;
+        veidSlotCount = 0;
+        availableSpanCount = 0;
+        totalSpanCount = 0;
+        count = 0;
+        for (i = pKernelMIGGpuInstance->resourceAllocation.veidOffset; i < GPUInstanceVeidEnd; i += veidStepSize)
+        {
+            // Determine max correctly sized VEID segments
+            if (((GPUInstanceFreeVeidMask >> i) & veidMask) == veidMask)
+            {
+                pParams->availableSpans[availableSpanCount].lo = count;
+                pParams->availableSpans[availableSpanCount].hi = count + (profile.veidCount / veidStepSize) - 1;
+                availableSpanCount++;
+            }
+
+            // Determine max correctly sized VEID segments
+            if (((GPUInstanceVeidMask >> i) & veidMask) == veidMask)
+            {
+                pParams->totalSpans[totalSpanCount].lo = count;
+                pParams->totalSpans[totalSpanCount].hi = count + (profile.veidCount / veidStepSize) - 1;
+                totalSpanCount++;
+            }
+
+            // Determine max correctly sized VEID segments
+            if (((GPUInstancePseudoMask >> i) & veidMask) == veidMask)
+            {
+                veidSlotCount++;
+                GPUInstancePseudoMask &= ~(veidMask << i);
+            }
+            count++;
+        }
+        pParams->totalProfileCount = NV_MIN(pKernelMIGGpuInstance->pProfile->virtualGpcCount / profile.gpcCount,
+                                                pKernelMIGGpuInstance->pProfile->veidCount / profile.veidCount);
+        pParams->totalSpansCount     = totalSpanCount;
+        pParams->profileCount        = veidSlotCount;
+        pParams->availableSpansCount = availableSpanCount;
+    }
+
+    return NV_OK;
+}
