@@ -326,6 +326,17 @@ _bar2WalkCBUpdatePde
             const NvU64               physAddr = memdescGetPhysAddr(pSubMemDesc, AT_GPU, 0);
 
             // Set fields within the temp PDE
+            if (pFmt->version == GMMU_FMT_VERSION_3)
+            {
+                NvU32 pdePcfHw  = 0;
+                NvU32 pdePcfSw  = 0;
+
+                pdePcfSw |= memdescGetVolatility(pSubMemDesc) ? (1 << SW_MMU_PCF_UNCACHED_IDX) : 0;
+                NV_ASSERT_OR_RETURN((kgmmuTranslatePdePcfFromSw_HAL(pKernelGmmu, pdePcfSw, &pdePcfHw) == NV_OK),
+                                      NV_ERR_INVALID_ARGUMENT);
+                nvFieldSet32(&pPde->fldPdePcf, pdePcfHw, entry.v8);
+            }
+            else
             {
                 nvFieldSetBool(&pPde->fldVolatile, memdescGetVolatility(pSubMemDesc), entry.v8);
             }
